@@ -76,7 +76,7 @@ function prettyDate(date){
                 day_diff < 31 && Math.ceil( day_diff / 7 ) + " weeks ago";
         day_diff >= 31 && Math.ceil( day_diff / 30 ) + " months ago";
     } catch(err) {
-        //console.log("Error: " + err);
+        console.log("Error: " + err);
         return "some time ago";
     }
 }
